@@ -5,8 +5,9 @@ from .IMotion import IMotion
 
 class IRoof(IMotion, metaclass=ABCMeta):
     """The module controls a roof."""
-    __module__ = 'pyobs.interfaces'
+
+    __module__ = "pyobs.interfaces"
     pass
 
 
-__all__ = ['IRoof']
+__all__ = ["IRoof"]
