@@ -1,13 +1,14 @@
 """
 TODO: write doc
 """
-__title__ = 'FITS utilities'
+__title__ = "FITS utilities"
 
 import logging
 import re
-from typing import Union
+from typing import Union, Any, cast, Optional, List, Callable, Dict
 import numpy as np
-from astropy.io.fits import Header
+from astropy.io import fits
+from numpy.typing import NDArray
 
 from pyobs.utils.time import Time
 
@@ -15,7 +16,7 @@ from pyobs.utils.time import Time
 log = logging.getLogger(__name__)
 
 
-def fitssec(hdu, keyword: str = 'TRIMSEC') -> np.ndarray:
+def fitssec(hdu: Any, keyword: str = "TRIMSEC") -> NDArray[Any]:
     """Trim an image to TRIMSEC or BIASSEC.
 
     Args:
@@ -29,26 +30,26 @@ def fitssec(hdu, keyword: str = 'TRIMSEC') -> np.ndarray:
     # keyword not given?
     if keyword not in hdu.header:
         # return whole data
-        return hdu.data
+        return cast(NDArray[Any], hdu.data)
 
     # get value of section
     sec = hdu.header[keyword]
 
     # split values
-    s = sec[1:-1].split(',')
-    x = s[0].split(':')
-    y = s[1].split(':')
+    s = sec[1:-1].split(",")
+    x = s[0].split(":")
+    y = s[1].split(":")
     x0 = int(x[0]) - 1
     x1 = int(x[1])
     y0 = int(y[0]) - 1
     y1 = int(y[1])
 
     # return data
-    return hdu.data[y0:y1, x0:x1]
+    return cast(NDArray[Any], hdu.data[y0:y1, x0:x1])
 
 
 class FilenameFormatter:
-    def __init__(self, fmt: Union[str, list], keys: dict = None):
+    def __init__(self, fmt: Union[str, List[str]], keys: Optional[Dict[str, Any]] = None):
         """Initializes a new filename formatter.
 
         Args:
@@ -56,23 +57,23 @@ class FilenameFormatter:
             keys: Additional keys to pass to the formatter.
         """
         self.format = fmt
-        self.keys = {} if keys is None else keys
+        self.keys: Dict[str, Any] = {} if keys is None else keys
 
         # define functions
-        self.funcs = {
-            'lower': self._format_lower,
-            'time': self._format_time,
-            'date': self._format_date,
-            'filter': self._format_filter,
-            'string': self._format_string,
-            'type': self._format_type
+        self.funcs: Dict[str, Callable[..., str]] = {
+            "lower": self._format_lower,
+            "time": self._format_time,
+            "date": self._format_date,
+            "filter": self._format_filter,
+            "string": self._format_string,
+            "type": self._format_type,
         }
 
-    def _value(self, hdr: Header, key: str):
+    def _value(self, hdr: fits.Header, key: str) -> Any:
         """Returns value for given key.
 
         Args:
-            hdr: Header to take value from.
+            hdr: fits.Header to take value from.
             key: Key to return value for.
 
         Returns:
@@ -86,7 +87,7 @@ class FilenameFormatter:
         # then check header
         return hdr[key]
 
-    def __call__(self, hdr: Header) -> str:
+    def __call__(self, hdr: fits.Header) -> Optional[str]:
         """Formats a filename given a format template and a FITS header.
 
         Args:
@@ -102,14 +103,14 @@ class FilenameFormatter:
         # make fmt a list
         if self.format is None:
             return None
-        if not isinstance(self.format, list):
+        if isinstance(self.format, str):
             self.format = [self.format]
 
         # loop formats
         for f in self.format:
             try:
                 # find all placeholders in format
-                placeholders = re.findall('\{[\w\d_-]+(?:\|[\w\d_-]+\:?(?:[\w\d_-]+)*)?\}', f)
+                placeholders = re.findall("\{[\w\d_-]+(?:\|[\w\d_-]+\:?(?:[\w\d_-]+)*)?\}", f)
                 if len(placeholders) == 0:
                     return f
 
@@ -129,9 +130,9 @@ class FilenameFormatter:
                 pass
 
         # still here?
-        raise KeyError('No valid format found.')
+        raise KeyError("No valid format found.")
 
-    def _format_placeholder(self, placeholder: str, hdr: Header) -> str:
+    def _format_placeholder(self, placeholder: str, hdr: fits.Header) -> str:
         """Format a given placeholder.
 
         Args:
@@ -151,16 +152,16 @@ class FilenameFormatter:
         params = []
 
         # do we have a pipe in here?
-        if '|' in key:
-            key, method = key.split('|')
+        if "|" in key:
+            key, method = key.split("|")
 
         # parameters for method?
-        if method is not None and ':' in method:
-            method, *params = method.split(':')
+        if method is not None and ":" in method:
+            method, *params = method.split(":")
 
         # if no method is given, just replace
         if method is None:
-            return self._value(hdr, key)
+            return str(self._value(hdr, key))
 
         else:
             # get function (may raise KeyError)
@@ -169,34 +170,34 @@ class FilenameFormatter:
             # call method and replace
             return func(hdr, key, *params)
 
-    def _format_lower(self, hdr: Header, key: str) -> str:
+    def _format_lower(self, hdr: fits.Header, key: str) -> str:
         """Sets a given string to lowercase.
 
-       Args:
-           hdr: FITS header to take values from.
-           key: The name of the FITS header key to use.
+        Args:
+            hdr: FITS header to take values from.
+            key: The name of the FITS header key to use.
 
-       Returns:
-           Formatted string.
-       """
-        return self._value(hdr, key).lower()
+        Returns:
+            Formatted string.
+        """
+        return str(self._value(hdr, key)).lower()
 
-    def _format_time(self, hdr: Header, key: str, delimiter: str = '-') -> str:
+    def _format_time(self, hdr: fits.Header, key: str, delimiter: str = "-") -> str:
         """Formats time using the given delimiter.
 
-       Args:
-           hdr: FITS header to take values from.
-           key: The name of the FITS header key to use.
-           delimiter: Delimiter for time formatting.
+        Args:
+            hdr: FITS header to take values from.
+            key: The name of the FITS header key to use.
+            delimiter: Delimiter for time formatting.
 
-       Returns:
-           Formatted string.
-       """
-        fmt = '%H' + delimiter + '%M' + delimiter + '%S'
+        Returns:
+            Formatted string.
+        """
+        fmt = "%H" + delimiter + "%M" + delimiter + "%S"
         date_obs = Time(self._value(hdr, key))
-        return date_obs.datetime.strftime(fmt)
+        return str(date_obs.datetime.strftime(fmt))
 
-    def _format_date(self, hdr: Header, key: str, delimiter: str = '-') -> str:
+    def _format_date(self, hdr: fits.Header, key: str, delimiter: str = "-") -> str:
         """Formats date using the given delimiter.
 
         Args:
@@ -207,11 +208,11 @@ class FilenameFormatter:
         Returns:
             Formatted string.
         """
-        fmt = '%Y' + delimiter + '%m' + delimiter + '%d'
+        fmt = "%Y" + delimiter + "%m" + delimiter + "%d"
         date_obs = Time(self._value(hdr, key))
-        return date_obs.datetime.strftime(fmt)
+        return str(date_obs.datetime.strftime(fmt))
 
-    def _format_filter(self, hdr: Header, key: str, image_type: str = 'IMAGETYP', prefix: str = '-') -> str:
+    def _format_filter(self, hdr: fits.Header, key: str, image_type: str = "IMAGETYP", prefix: str = "-") -> str:
         """Formats a filter, prefixed by a given separator, only if the image type requires it.
 
         Args:
@@ -224,26 +225,26 @@ class FilenameFormatter:
             Formatted string.
         """
         it = hdr[image_type].lower()
-        if it in ['light', 'object', 'flat', 'skyflat']:
-            return prefix + self._value(hdr, key)
+        if it in ["light", "object", "flat", "skyflat"]:
+            return str(prefix + self._value(hdr, key))
         else:
-            return ''
+            return ""
 
-    def _format_string(self, hdr: Header, key: str, format: str) -> str:
+    def _format_string(self, hdr: fits.Header, key: str, fmt: str) -> str:
         """Formats a string using Python string substitution.
 
         Args:
             hdr: FITS header to take values from.
             key: The name of the FITS header key to use.
-            format: A Python string format like %d, %05d, or %4.1f.
+            fmt: A Python string format like %d, %05d, or %4.1f.
 
         Returns:
             Formatted string.
         """
-        fmt = '%' + format
-        return fmt % self._value(hdr, key)
+        fmt = "%" + fmt
+        return str(fmt % self._value(hdr, key))
 
-    def _format_type(self, hdr: Header, key: str) -> str:
+    def _format_type(self, hdr: fits.Header, key: str) -> str:
         """Formats an image type to a one-letter code.
 
         Args:
@@ -253,17 +254,19 @@ class FilenameFormatter:
         Returns:
             Formatted string.
         """
-        if self._value(hdr, key) == 'bias':
-            return 'b'
-        elif self._value(hdr, key) == 'skyflat':
-            return 'f'
-        elif self._value(hdr, key) == 'dark':
-            return 'd'
+        if self._value(hdr, key) == "bias":
+            return "b"
+        elif self._value(hdr, key) == "skyflat":
+            return "f"
+        elif self._value(hdr, key) == "dark":
+            return "d"
         else:
-            return 'e'
+            return "e"
 
 
-def format_filename(hdr: Header, fmt: Union[str, list], keys: dict = None) -> str:
+def format_filename(
+    hdr: fits.Header, fmt: Union[str, List[str]], keys: Optional[Dict[str, Any]] = None
+) -> Optional[str]:
     """Formats a filename given a format template and a FITS header.
 
     Args:
@@ -282,4 +285,4 @@ def format_filename(hdr: Header, fmt: Union[str, list], keys: dict = None) -> st
     return ff(hdr)
 
 
-__all__ = ['format_filename', 'FilenameFormatter']
+__all__ = ["format_filename", "FilenameFormatter", "fitssec"]
